@@ -22,7 +22,6 @@
 #define __GTK_FILE_SYSTEM_H__
 
 #include <gio/gio.h>
-#include <glib-object.h>
 #include <gtk/gtkwidget.h>	/* For icon handling */
 
 G_BEGIN_DECLS
@@ -34,6 +33,11 @@ G_BEGIN_DECLS
 #define GTK_IS_FILE_SYSTEM_CLASS(c)  (G_TYPE_CHECK_CLASS_TYPE    ((c), GTK_TYPE_FILE_SYSTEM))
 #define GTK_FILE_SYSTEM_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS  ((o), GTK_TYPE_FILE_SYSTEM, GtkFileSystemClass))
 
+typedef struct GtkFileSystem          GtkFileSystem;
+typedef struct GtkFileSystemPrivate      GtkFileSystemPrivate;
+typedef struct GtkFileSystemClass     GtkFileSystemClass;
+
+
 #define GTK_TYPE_FOLDER         (_gtk_folder_get_type ())
 #define GTK_FOLDER(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GTK_TYPE_FOLDER, GtkFolder))
 #define GTK_FOLDER_CLASS(c)     (G_TYPE_CHECK_CLASS_CAST    ((c), GTK_TYPE_FOLDER, GtkFolderClass))
@@ -41,30 +45,21 @@ G_BEGIN_DECLS
 #define GTK_IS_FOLDER_CLASS(c)  (G_TYPE_CHECK_CLASS_TYPE    ((c), GTK_TYPE_FOLDER))
 #define GTK_FOLDER_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS  ((o), GTK_TYPE_FOLDER, GtkFolderClass))
 
-/* GError enumeration for GtkFileSystem
- */
+typedef struct GtkFolder          GtkFolder;
+typedef struct GtkFolderPrivate      GtkFolderPrivate;
+typedef struct GtkFolderClass     GtkFolderClass;
 
-#define GTK_FILE_SYSTEM_ERROR (_gtk_file_system_error_quark ())
-
-typedef enum
-{
-  GTK_FILE_SYSTEM_ERROR_NONEXISTENT,
-  GTK_FILE_SYSTEM_ERROR_NOT_FOLDER,
-  GTK_FILE_SYSTEM_ERROR_INVALID_URI,
-  GTK_FILE_SYSTEM_ERROR_BAD_FILENAME,
-  GTK_FILE_SYSTEM_ERROR_FAILED,
-  GTK_FILE_SYSTEM_ERROR_ALREADY_EXISTS
-} GtkFileSystemError;
-
-GQuark     _gtk_file_system_error_quark      (void);
-
-typedef struct GtkFileSystemClass GtkFileSystemClass;
-typedef struct GtkFileSystem GtkFileSystem;
-typedef struct GtkFolderClass GtkFolderClass;
-typedef struct GtkFolder GtkFolder;
 typedef struct GtkFileSystemVolume GtkFileSystemVolume; /* opaque struct */
 typedef struct GtkFileSystemBookmark GtkFileSystemBookmark; /* opaque struct */
 
+
+struct GtkFileSystem
+{
+  GObject parent_object;
+
+  GtkFileSystemPrivate *priv;
+};
+
 struct GtkFileSystemClass
 {
   GObjectClass parent_class;
@@ -73,9 +68,12 @@ struct GtkFileSystemClass
   void (*volumes_changed)   (GtkFileSystem *file_system);
 };
 
-struct GtkFileSystem
+
+struct GtkFolder
 {
   GObject parent_object;
+
+  GtkFolderPrivate *priv;
 };
 
 struct GtkFolderClass
@@ -92,10 +90,6 @@ struct GtkFolderClass
   void (*deleted)          (GtkFolder *folder);
 };
 
-struct GtkFolder
-{
-  GObject parent_object;
-};
 
 typedef void (* GtkFileSystemGetFolderCallback)    (GCancellable        *cancellable,
 						    GtkFolder           *folder,
@@ -180,7 +174,8 @@ GdkPixbuf *           _gtk_file_system_volume_render_icon      (GtkFileSystemVol
 							        gint                  icon_size,
 							        GError              **error);
 
-void                  _gtk_file_system_volume_free             (GtkFileSystemVolume *volume);
+GtkFileSystemVolume  *_gtk_file_system_volume_ref              (GtkFileSystemVolume *volume);
+void                  _gtk_file_system_volume_unref            (GtkFileSystemVolume *volume);
 
 /* GtkFileSystemBookmark methods */
 void                   _gtk_file_system_bookmark_free          (GtkFileSystemBookmark *bookmark);
@@ -190,6 +185,8 @@ GdkPixbuf *     _gtk_file_info_render_icon (GFileInfo *info,
 					    GtkWidget *widget,
 					    gint       icon_size);
 
+gboolean	_gtk_file_info_consider_as_directory (GFileInfo *info);
+
 G_END_DECLS
 
 #endif /* __GTK_FILE_SYSTEM_H__ */
