@@ -21,12 +21,13 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
 #include "config.h"
 
-#include "gtkmenu.h"
+#include "gtkmenuprivate.h"
+#include "gtkmenuitemprivate.h"
 #include "gtktearoffmenuitem.h"
 #include "gtkintl.h"
 
@@ -39,13 +40,17 @@ struct _GtkTearoffMenuItemPrivate
   guint torn_off : 1;
 };
 
-static void gtk_tearoff_menu_item_size_request (GtkWidget             *widget,
-				                GtkRequisition        *requisition);
-static gboolean gtk_tearoff_menu_item_draw   (GtkWidget             *widget,
-					      cairo_t               *cr);
-static void gtk_tearoff_menu_item_activate   (GtkMenuItem           *menu_item);
-static void gtk_tearoff_menu_item_parent_set (GtkWidget             *widget,
-					      GtkWidget             *previous);
+static void gtk_tearoff_menu_item_get_preferred_width  (GtkWidget      *widget,
+                                                        gint           *minimum,
+                                                        gint           *natural);
+static void gtk_tearoff_menu_item_get_preferred_height (GtkWidget      *widget,
+                                                        gint           *minimum,
+                                                        gint           *natural);
+static gboolean gtk_tearoff_menu_item_draw             (GtkWidget      *widget,
+                                                        cairo_t        *cr);
+static void gtk_tearoff_menu_item_activate             (GtkMenuItem    *menu_item);
+static void gtk_tearoff_menu_item_parent_set           (GtkWidget      *widget,
+                                                        GtkWidget      *previous);
 
 G_DEFINE_TYPE (GtkTearoffMenuItem, gtk_tearoff_menu_item, GTK_TYPE_MENU_ITEM)
 
@@ -65,7 +70,8 @@ gtk_tearoff_menu_item_class_init (GtkTearoffMenuItemClass *klass)
   menu_item_class = (GtkMenuItemClass*) klass;
 
   widget_class->draw = gtk_tearoff_menu_item_draw;
-  widget_class->size_request = gtk_tearoff_menu_item_size_request;
+  widget_class->get_preferred_width = gtk_tearoff_menu_item_get_preferred_width;
+  widget_class->get_preferred_height = gtk_tearoff_menu_item_get_preferred_height;
   widget_class->parent_set = gtk_tearoff_menu_item_parent_set;
 
   menu_item_class->activate = gtk_tearoff_menu_item_activate;
@@ -87,27 +93,53 @@ gtk_tearoff_menu_item_init (GtkTearoffMenuItem *tearoff_menu_item)
 }
 
 static void
-gtk_tearoff_menu_item_size_request (GtkWidget      *widget,
-				    GtkRequisition *requisition)
+gtk_tearoff_menu_item_get_preferred_width (GtkWidget      *widget,
+                                           gint           *minimum,
+                                           gint           *natural)
 {
-  GtkStyle *style;
+  GtkStyleContext *context;
+  guint border_width;
+  GtkBorder padding;
+  GtkStateFlags state;
+
+  context = gtk_widget_get_style_context (widget);
+  state = gtk_widget_get_state_flags (widget);
+
+  gtk_style_context_get_padding (context, state, &padding);
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+
+  *minimum = *natural = (border_width + BORDER_SPACING) * 2 + padding.left + padding.right;
+}
+
+static void
+gtk_tearoff_menu_item_get_preferred_height (GtkWidget      *widget,
+                                            gint           *minimum,
+                                            gint           *natural)
+{
+  GtkStyleContext *context;
+  GtkBorder padding;
+  GtkStateFlags state;
   GtkWidget *parent;
   guint border_width;
 
-  style = gtk_widget_get_style (widget);
+  context = gtk_widget_get_style_context (widget);
+  state = gtk_widget_get_state_flags (widget);
 
+  gtk_style_context_get_padding (context, state, &padding);
   border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-  requisition->width = (border_width + style->xthickness + BORDER_SPACING) * 2;
-  requisition->height = (border_width + style->ythickness) * 2;
+
+  *minimum = *natural = (border_width * 2) + padding.top + padding.bottom;
 
   parent = gtk_widget_get_parent (widget);
-  if (GTK_IS_MENU (parent) && GTK_MENU (parent)->torn_off)
+  if (GTK_IS_MENU (parent) && GTK_MENU (parent)->priv->torn_off)
     {
-      requisition->height += ARROW_SIZE;
+      *minimum += ARROW_SIZE;
+      *natural += ARROW_SIZE;
     }
   else
     {
-      requisition->height += style->ythickness + 4;
+      *minimum += padding.top + 4;
+      *natural += padding.top + 4;
     }
 }
 
@@ -116,22 +148,20 @@ gtk_tearoff_menu_item_draw (GtkWidget *widget,
                             cairo_t   *cr)
 {
   GtkMenuItem *menu_item;
-  GtkShadowType shadow_type;
-  GtkStateType state;
-  GtkStyle *style;
+  GtkStateFlags state;
+  GtkStyleContext *context;
+  GtkBorder padding;
   gint x, y, width, height;
   gint right_max;
   guint border_width;
-  GtkArrowType arrow_type;
   GtkTextDirection direction;
   GtkWidget *parent;
-  GdkWindow *window;
+  gdouble angle;
 
   menu_item = GTK_MENU_ITEM (widget);
-  style = gtk_widget_get_style (widget);
-  window = gtk_widget_get_window (widget);
+  context = gtk_widget_get_style_context (widget);
   direction = gtk_widget_get_direction (widget);
-  state = gtk_widget_get_state (widget);
+  state = gtk_widget_get_state_flags (widget);
 
   border_width = gtk_container_get_border_width (GTK_CONTAINER (menu_item));
   x = border_width;
@@ -140,84 +170,78 @@ gtk_tearoff_menu_item_draw (GtkWidget *widget,
   height = gtk_widget_get_allocated_height (widget) - border_width * 2;
   right_max = x + width;
 
-  if (state == GTK_STATE_PRELIGHT)
+  gtk_style_context_save (context);
+  gtk_style_context_set_state (context, state);
+  gtk_style_context_get_padding (context, state, &padding);
+
+  if (state & GTK_STATE_FLAG_PRELIGHT)
     {
-      gint selected_shadow_type;
-      
-      gtk_widget_style_get (widget,
-                            "selected-shadow-type", &selected_shadow_type,
-                            NULL);
-      gtk_paint_box (style,
-                     cr,
-                     GTK_STATE_PRELIGHT,
-                     selected_shadow_type,
-                     widget, "menuitem",
-                     x, y, width, height);
+      gtk_render_background (context, cr, x, y, width, height);
+      gtk_render_frame (context, cr, x, y, width, height);
     }
 
   parent = gtk_widget_get_parent (widget);
-  if (GTK_IS_MENU (parent) && GTK_MENU (parent)->torn_off)
+  if (GTK_IS_MENU (parent) && GTK_MENU (parent)->priv->torn_off)
     {
       gint arrow_x;
 
-      if (state == GTK_STATE_PRELIGHT)
-        shadow_type = GTK_SHADOW_IN;
-      else
-        shadow_type = GTK_SHADOW_OUT;
-
-      if (menu_item->toggle_size > ARROW_SIZE)
+      if (menu_item->priv->toggle_size > ARROW_SIZE)
         {
-          if (direction == GTK_TEXT_DIR_LTR) {
-            arrow_x = x + (menu_item->toggle_size - ARROW_SIZE)/2;
-            arrow_type = GTK_ARROW_LEFT;
-          }
-          else {
-            arrow_x = x + width - menu_item->toggle_size + (menu_item->toggle_size - ARROW_SIZE)/2; 
-            arrow_type = GTK_ARROW_RIGHT;	    
-          }
-          x += menu_item->toggle_size + BORDER_SPACING;
+          if (direction == GTK_TEXT_DIR_LTR)
+            {
+              arrow_x = x + (menu_item->priv->toggle_size - ARROW_SIZE)/2;
+              angle = (3 * G_PI) / 2;
+            }
+          else
+            {
+              arrow_x = x + width - menu_item->priv->toggle_size + (menu_item->priv->toggle_size - ARROW_SIZE)/2;
+              angle = G_PI / 2;
+            }
+          x += menu_item->priv->toggle_size + BORDER_SPACING;
         }
       else
         {
-          if (direction == GTK_TEXT_DIR_LTR) {
-            arrow_x = ARROW_SIZE / 2;
-            arrow_type = GTK_ARROW_LEFT;
-          }
-          else {
-            arrow_x = x + width - 2 * ARROW_SIZE + ARROW_SIZE / 2; 
-            arrow_type = GTK_ARROW_RIGHT;	    
-          }
+          if (direction == GTK_TEXT_DIR_LTR)
+            {
+              arrow_x = ARROW_SIZE / 2;
+              angle = (3 * G_PI) / 2;
+            }
+          else
+            {
+              arrow_x = x + width - 2 * ARROW_SIZE + ARROW_SIZE / 2;
+              angle = G_PI / 2;
+            }
           x += 2 * ARROW_SIZE;
         }
 
-
-      gtk_paint_arrow (style, cr,
-                       state, shadow_type,
-                       widget, "tearoffmenuitem",
-                       arrow_type, FALSE,
-                       arrow_x, y + height / 2 - 5, 
-                       ARROW_SIZE, ARROW_SIZE);
+      gtk_render_arrow (context, cr, angle,
+                        arrow_x, height / 2 - 5,
+                        ARROW_SIZE);
     }
 
   while (x < right_max)
     {
       gint x1, x2;
 
-      if (direction == GTK_TEXT_DIR_LTR) {
-        x1 = x;
-        x2 = MIN (x + TEAR_LENGTH, right_max);
-      }
-      else {
-        x1 = right_max - x;
-        x2 = MAX (right_max - x - TEAR_LENGTH, 0);
-      }
+      if (direction == GTK_TEXT_DIR_LTR)
+        {
+          x1 = x;
+          x2 = MIN (x + TEAR_LENGTH, right_max);
+        }
+      else
+        {
+          x1 = right_max - x;
+          x2 = MAX (right_max - x - TEAR_LENGTH, 0);
+        }
 
-      gtk_paint_hline (style, cr, GTK_STATE_NORMAL,
-                       widget, "tearoffmenuitem",
-                       x1, x2, y + (height - style->ythickness) / 2);
+      gtk_render_line (context, cr,
+                       x1, y + (height - padding.bottom) / 2,
+                       x2, y + (height - padding.bottom) / 2);
       x += 2 * TEAR_LENGTH;
     }
 
+  gtk_style_context_restore (context);
+
   return FALSE;
 }
 
@@ -233,14 +257,14 @@ gtk_tearoff_menu_item_activate (GtkMenuItem *menu_item)
 
       gtk_widget_queue_resize (GTK_WIDGET (menu_item));
       gtk_menu_set_tearoff_state (GTK_MENU (parent),
-				  !menu->torn_off);
+                                  !menu->priv->torn_off);
     }
 }
 
 static void
 tearoff_state_changed (GtkMenu            *menu,
-		       GParamSpec         *pspec,
-		       gpointer            data)
+                       GParamSpec         *pspec,
+                       gpointer            data)
 {
   GtkTearoffMenuItem *tearoff_menu_item = GTK_TEAROFF_MENU_ITEM (data);
   GtkTearoffMenuItemPrivate *priv = tearoff_menu_item->priv;
@@ -250,7 +274,7 @@ tearoff_state_changed (GtkMenu            *menu,
 
 static void
 gtk_tearoff_menu_item_parent_set (GtkWidget *widget,
-				  GtkWidget *previous)
+                                  GtkWidget *previous)
 {
   GtkTearoffMenuItem *tearoff_menu_item = GTK_TEAROFF_MENU_ITEM (widget);
   GtkTearoffMenuItemPrivate *priv = tearoff_menu_item->priv;
@@ -261,15 +285,15 @@ gtk_tearoff_menu_item_parent_set (GtkWidget *widget,
   menu = GTK_IS_MENU (parent) ? GTK_MENU (parent) : NULL;
 
   if (previous)
-    g_signal_handlers_disconnect_by_func (previous, 
-					  tearoff_state_changed, 
-					  tearoff_menu_item);
-  
+    g_signal_handlers_disconnect_by_func (previous,
+                                          tearoff_state_changed,
+                                          tearoff_menu_item);
+
   if (menu)
     {
       priv->torn_off = gtk_menu_get_tearoff_state (menu);
-      g_signal_connect (menu, "notify::tearoff-state", 
-			G_CALLBACK (tearoff_state_changed), 
-			tearoff_menu_item);
-    }  
+      g_signal_connect (menu, "notify::tearoff-state",
+                        G_CALLBACK (tearoff_state_changed),
+                        tearoff_menu_item);
+    }
 }
